function TabTeX(file,title,label,chead,hrow,fcol,hline,csin,dec,data,fnote,fsty,mpage,stat,ihead,itext);
%
% 1-25, 2006 (c) Lukasz Drozd, Jaromir Nosal
% Creates LaTEX code of a table according to specifications.
% For available layouts see template description below. Uses LaTEX package 
% booktabs.sty. 
%==========================================================================
%
% 1. TABLE TEMPLATE LAYOUT:
%
% TITLE
% ___________________________________________________________________
%     ROW 1       TABLE HEADING (NUMBER OF ROWS AND MULTICOLUMN
%                                               STRUCTURE OPTIONAL)*
%               ---------------------------  ------------------------
%     ROW 2   TABLE HEADING                         TABLE HEADING  
% -------------------------------------------------------------------
%                0.93   0.70   0.73   0.44    90   113   43
%                0.78   0.58   0.91   0.57    75    80   51
%                0.63   0.56   0.86   0.77    30    66   53
%                0.90                                    47
%    FIRST               TABLE BODY (NUMERIC DATA ONLY) 
%    COLUMN                  
%    LABELS      0.58   0.                          64   55 
%  (OPTIONAL)    0.87   0.88   0.87   0.72    35    95   60
% --------------- ADDITIONAL HORIZONTAL LINES IF SPECIFIED (hline field)
% ---------ADDITIONAL HORIZONTAL LABELS IF SPECIFIED (ihead,itext field)
%                0.88   0.92   0.87   0.72    35    78   50
%                0.94   0.77   0.80   0.14   102   104   37
%                0.74   0.41   0.72   0.68    46    72   49
% -----------------------LINE AUTOMATICALLY ADDED--------------------
%    MEDIAN      0.87   0.63   0.80   0.60    45    80   50  
%   (OPTIONAL)
% ___________________________________________________________________
% NOTES: (script size table footnote)
% (*) - Horizontal lines are generated automatically, if multicolumn 
%       specification used.
%       Lines in heading can be suppressed 
%           - in chead use 0 instead of 2 whenever applicable.
%==========================================================================
%
% 2. ARGUMENT SPECIFICATION:
%
%--------------------------------------------------------------------------
% Entry | Description                | Type                   | Example   
%--------------------------------------------------------------------------
%       | Name of the output file    | cell array of strings  | {'Tab.tex'}
%       | with resulting TeX code.   | 1 row                  | 
% file  | The subfolder \Tex must    |                        |  or
%       | exist in the current       | output file writen to  | 
%       | MATLAB workspace.          | \Tex subfolder         | {'Tab.txt'}
%--------------------------------------------------------------------------
%       | Title of the table         | cell array of strings  |
%       | environment - apears in    | 1 row                  |
% title | caption field of the       |                        | {'CA deficit'}
%       | 'TeX table environment'    |                        |
%--------------------------------------------------------------------------
%       | Table label, apears        | cell array of strings  |
% label | in the label field         | 1 row                  | {'Table3'}
%       | of 'TeX table environment' |                        |
%--------------------------------------------------------------------------
%       | Table heading specification| matrix with integers   | [0 1 2 2 2;
%       | - matrix specifying heading| number of rows=        |  1 1 1 1 1]
% chead*| layout. Can be suppressed. | number of heading rows |      
%       | 0 - no label in this cell  | number of columns =    |     or
%       | 1 - add label from hrow    | number of columns in   | [] -BLANK 
%       | 2 - merge with previous    | the entire table       |  
%       | See (*) below for details  |                        | 
%--------------------------------------------------------------------------
%       | Table heading labels (TEX) | cell array of strings: | {'$x^{T}$';'$y$'}
%       | - each subsequent entry    | number of fields must  | 
% hrow  | corresponds to subsequent  | agree with the number  |   or
%       | 1s in the chead matrix     | of 1s entered in chead |   
%       | Heading suppressed if BLANK| Read sequentially.     | {} -BLANK
%--------------------------------------------------------------------------
%       | First column non-          |                        | {'BEL';'CAN'}
%       | numeric data/labels.       | cell array of strings  | 
% fcol  | Suppressed if BLANK.       |                        |     or
%       |                            |                        | {} -BLANK
%--------------------------------------------------------------------------
%       | Specifies additional       | vector row numbers     | [2;3]
% hline | horizontal lines           | in ascending order     | adds horiz.
%       | to apear after the         | can't exceed number    | line after
%       | indicated rows             | of rows in the table   | rows 2,3 
%--------------------------------------------------------------------------
%       | Size of each column in     | numeric                | 
% csin  | inches                     | vector                 | [2;1.1;0.5]
%       |                            |                        |
%--------------------------------------------------------------------------
%       | Number of deciman places   | numeric integers       | 
% dec   | in each column of data     | vector                 | [2;1]
%       |                            |                        |
%--------------------------------------------------------------------------
%       | Matrix with numeric data   | data(table rows,       |
% data  | needs to be formatted (num.|         table columns) | [0.8 0.2;
%       | of decimal digits adjusted)|                        |  1.1 2.0]
%       | See example call below.    |                        |  
%--------------------------------------------------------------------------
%       | Text of the footnote under | cell array of strings  | {'HP filtered'}
% fnote | in script size the table   |                        |     or
%       | Suppressed if blank {}     |                        | {} -BLANK
%--------------------------------------------------------------------------
%       | =1 footnote in Italics     | numeric integer        |
% fsty  | =0 no Italics              | 0,1                    | 1
%       | Suppressed if blank {}     |                        | 
%--------------------------------------------------------------------------
%       | Sets up minipage           | numeric vector         | [0]
%       | format environment for the | 1 row, 1 column        | 
% mpage | table. Suppressed if BLANK.|                        | or
%       | 0  - autosize              |                        |
%       | >0 - manual size spec.     |                        | []-BLANK
%--------------------------------------------------------------------------
%       | =2 adds the last row with  |                        |
%       |  column averages generated |                        |
%       |  automatically             |  numeric integer       |
% stat  | =1 adds the last row with  |  0,1,2                 |  2
%       |  column medians generated  |                        |  
%       |  automatically,            |                        |
%       | =0 no median or average    |                        |
%--------------------------------------------------------------------------
% OPTIONAL PARAMETERS (can be dropped from function call)
%--------------------------------------------------------------------------
%       | Defines rows with labels   | vector,                |
%       | inside the table.          | of integers            |
%       |                            | specifies              |
% ihead |                            | row position of the    |  [2;3];
%       |                            | label - to appear      |   
%       |                            | after the data row     |
%       |                            | of the specified number|
%--------------------------------------------------------------------------
%       | text of the labels in      | cell array of strings  | {'label1';
% itext | ihead,                     | must be specified      | 'label2'}
%       | in order of appearance     | if ihead specified     |
%--------------------------------------------------------------------------
%(*) chead - Table heading layout specification:
%            Number of rows  determines the number of rows to be included in 
%            the heading of the table.
%            Number of columns must agree with the number of
%            columns in the entire table. 
%            Each entry of 'chead' specifies labeling options of the 
%            heading cells. Heading can be suppressed if left BLANK: chead=[];.
%
%            Entries in chead can take integer values from 0 to 2 :
%             - 1 inputs subsequent label from hrow, 0 leaves cell blank, 
%             - 2 builds a multicolumn object that extends previous labels
%               to all subsequent cells with this entry (merge current cell
%               with previous and extend the label throughout). 
%
%            Supporting horizontal lines are created automatically - see 
%            table template description how to suppress this option. 
%==========================================================================
% 3. EXAMPLE CALL
%
% (copy-paste to .m file and run, 
%  functions TabTeX, PrevTeX must be present in the current workspace,
%  Tex subfolder exist the current Matlab workspace)
% 
% clc; clear
% file = {'Sample.txt'};
% title= {'Here goes the title.'};
% label= {'Sample'};
% chead= [0 1 2 2 2 1 2 2;
%         1 1 1 1 1 1 1 1];
% hrow = {'Correlation','Relative volatility (\%)',...
%         'Area','$p_{x},p_{m}$','$p_{x},x$','$p_{m},x$','$p,x$',...
%         '$p_{x}$','$p_{m}$','$p$'};
% data = ones(9,7);
% ihead=[1,4];
% itext={'\textit{Green countries}','\textit{Blue countries}'};
% fcol = {'\quad Belgium \footnote{A footnote to the label.}';'\quad Canada';'\quad Switzerland';'\quad France';...
%           '\quad Germany';'\quad Italy';'\quad Japan';'\quad Netherlands';'\quad US'};   
% hline= [3];
% csin = [1.0;.5;.5;.5;.5;.5;.5;.5]; 
% dec  = [     2; 2; 2; 2; 1; 1; 1]; 
% stat=1; fsty=0; mpage=[0];
% fnote= {'Based on logged \& HP filtered quarterly data for the period 1980:1-2000:2. Sources listed at the end of the tables.'};
% TabTeX(file,title,label,chead,hrow,fcol,hline,csin,dec,data,fnote,fsty,mpage,stat,ihead,itext);
% PrevTeX({'Sample.txt'})
%==========================================================================
% 4. COMMON PROBLEMS: 
%
% a. In the strings must NOT use TeX restricted symbols '%,$,&...', 
%    unless proceeded with '\'. Causes LaTeX compilation Error.  
% b. PrevTeX function: pdflatex and Adobe Acrobat Reader must be 
%    installed and added to system path specification, 
%    Acrobat Reader must be closed.
% c. Tex subfolder in the current Matlab workspace must be created.
% d. In order to format notes properly use minipage environment
%    manually or specify the table width equal to page width. 
%


cd Tex %enter into Tex subfolder (must exist!!!)
cc  = size(data); %determine size specifications, check consistency
mm  = size(fcol);
hh  = size(chead);
dd  = size(dec);
ss  = size(csin);

if ~iscell(fcol) %check for errors
    cd ..
    error('TabTeX.m ERROR: Illegal specification in ''fcol'' ')
end

if isempty(fcol) %first column specification
    fsup=0; cc(1,2)=cc(1,2);cc(1,1)=cc(1,1)+min(stat,1)*1; %reset size specification if fcol empty
    %fsup =0 the first non-numeric data column is suppressed
else  
    if mm(1,1)~=cc(1,1)
        cd ..
        error('TabTeX.m ERROR: Size of fcols and data must agree')
    end
    fsup=1; cc(1,2)=cc(1,2)+1;cc(1,1)=cc(1,1)+min(stat,1)*1;
end 

if cc(1,2)~=max(dd)+1*fsup | cc(1,2)~=max(ss) | hh(1,2)~=max(dd)+1*fsup %column specification consistency check
   cd ..
   error('TabTeX.m ERROR: Inconsistent column specification in ''csin'', ''dec'' and ''chead'' ')
end

if isnumeric(dec) & isnumeric(data)
else
    cd ..
    error('TabTeX.m ERROR: Illegal specification in ''data/dec'' ')
end


%define the position of user requested interior labels
txt=zeros(cc(1,1),1);
if nargin == 14
    ext=0;
elseif nargin == 16
    ext=1;
    if ~isempty(ihead)
        tx=max(size(ihead)); counter=1;
        while( counter<=tx )
            for i=1:cc(1,1)-1*(1-min(stat,1))
                 if ihead(counter)==i 
                    txt(i)=1; 
                 end  
            end
            counter=counter+1;
        end
    end
else
    error('TabTeX.m ERROR: Invalid number of arguments')
end

%read user requested horizontal lines (hline)
hl=zeros(cc(1,1),1);
if ~isempty(hline)
   tx=max(size(hline)); counter=1;
   while( counter<=tx )
    for i=1:cc(1,1)-1*(1-min(stat,1))
         if hline(counter)==i 
             hl(i)=1; 
         end  
    end
   counter=counter+1;
   end
end

%size of the table
% 
%cc(1,1) num of rows excluding table heading, 
%cc(1,2) num of columns (including fcol unless suppressed by user)
%

if iscell(file) & iscell(title) & iscell(label) & isnumeric(csin) %check errors
else
    cd ..
     error('TabTeX.m ERROR: Illegal specification in ''file/title/label'' ')
end

%generate TeX code
% - heading
fid = fopen(char(file(1)),'w');
fprintf(fid,'\\begin{table}\n');

%set minipage environment for small tables 
if ~isempty(mpage)
    switch mpage(1)
        case 0
            fprintf(fid,'\\centering\n');
            fprintf(fid,'\\begin{minipage}[b]{%.2f\\linewidth}\n',(sum(csin)+.162*cc(1,2))/6.0);
        otherwise
            fprintf(fid,'\\centering\n');
            fprintf(fid,'\\begin{minipage}[b]{%.2f\\linewidth}\n',mpage(1));
        end
end

fprintf(fid,'\\label{%s}\n'  , char(label(1)));
fprintf(fid,'\\caption{%s}\n', char(title(1)));
fprintf(fid,'\\begin{tabular}{p{%gin}',csin(1));

for i=2:cc(1,2)
    fprintf(fid,'p{%gin}',csin(i));
end
fprintf(fid,'} \n \\toprule \n');
cursor=1;

if ~isempty(chead) & ~isempty(hrow)
stop   =zeros(cc(1,2),1);
start  =zeros(cc(1,2),1);
mc=1;
for i=1:hh(1,1)
    for j=1:cc(1,2)-1
        switch chead(i,j)
        case 0
            fprintf(fid,' & ');
        case 1
            if chead(i,j+1)==0 | chead(i,j+1)==1 
                        %enter if and only if subsequent entry is 
                        %0 (no label) or 1 (put label)
               fprintf(fid,'%s & ',char(hrow(cursor)));
               cursor=cursor+1; %cursor in hrow
             end 
        case 2
            if chead(i,j-1)==1
                mc=1;
                for z=j:cc(1,2)
                    if chead(i,z)==2
                       mc=mc+1;
                    else 
                       break
                    end
                end
                if j-1+mc-1==cc(1,2)
                    fprintf(fid,'\\multicolumn{%g}{l}{%s}',mc,char(hrow(cursor)));
                else
                    fprintf(fid,'\\multicolumn{%g}{l}{%s} & ',mc,char(hrow(cursor)));
                end
                start(j)=j-1; stop(j)=j-1+mc-1;
                %keep track of the multicolumn structure to generate horizontal
                %lines later
                cursor=cursor+1;
            end
        otherwise
            cd ..
              error('TabTeX.m ERROR: Illegal specification in ''chead'' ')
              
        end %end of switch
    end %end of inner for
    
    %deal with special case of the last two cells merged
    switch chead(i,cc(1,2))
        case 1
            fprintf(fid,'%s',char(hrow(cursor)));
            cursor=cursor+1;
        case 2
            if chead(i,cc(1,2)-1)==1
                mc=2;
                fprintf(fid,'\\multicolumn{%g}{l}{%s}',mc,char(hrow(cursor)));
                start(cc(1,2))=cc(1,2)-1; stop(cc(1,2))=cc(1,2);
                %keep track of multicolumn to generate horizontal
                %lines later
                cursor=cursor+1;
            end
        otherwise
              cd ..
              error('TabTeX.m ERROR: Illegal specification in ''chead'' ')
        end 
    fprintf(fid,'\\\\ \n ')
    
    %now add horizontal lines to the heading
    if mc>1
        for j=1:cc(1,2)
            if start(j)~=stop(j)
                fprintf(fid,'\\cmidrule(r){%g-%g} \n',start(j),stop(j));
            end
        end
    end

    start=0; %reset midrule cursors for the next row of headings
    stop =0;
    mc   =1;
end %code of the table heading generated (outer loop ends here)
fprintf(fid,'\\midrule \n');
end
cursor=1;
% - body (data)
switch stat
    
    case 0
        
        if fsup==0
            for i=1:cc(1,1)
                if txt(i)==1
                    fprintf(fid,'\n \\multicolumn{%g}{l}{%s} \\\\ \n',cc(1,2)-1,char(itext(cursor)));
                    cursor=cursor+1;
                end
                FMT=char(['%.', num2str(dec(1)),'f']);
                fprintf(fid,FMT,data(i,1));
            for j=2:cc(1,2)
                FMT=char(['& %.', num2str(round(dec(j))),'f']);
                fprintf(fid,FMT,data(i,j));
            end
            fprintf(fid,'\\\\ \n ');
            if hl(i)==1 %add horizontal line if requested by user
                fprintf(fid,'\\midrule \n');
            end
            end
        else
            for i=1:cc(1,1)
                if txt(i)==1
                    fprintf(fid,'\n \\multicolumn{%g}{l}{%s} \\\\ \n',cc(1,2)-1,char(itext(cursor)));
                    cursor=cursor+1;
                end
                fprintf(fid,'%s ',char(fcol(i)));
            for j=1:cc(1,2)-1
                FMT=char(['& %.', num2str(round(dec(j))),'f']);
                fprintf(fid,FMT,data(i,j));
            end
            fprintf(fid,'\\\\ \n ');
            if hl(i)==1 %add horizontal line if requested by user
                fprintf(fid,'\\midrule \n');
            end    
            end         
        end
        
    case 1
        
        if fsup==0
            for i=1:cc(1,1)-1
                if txt(i)==1
                    fprintf(fid,'\n \\multicolumn{%g}{l}{%s} \\\\ \n',cc(1,2)-1,char(itext(cursor)));
                    cursor=cursor+1;
                end
                FMT=char(['%.', num2str(round(dec(1))),'f']);
                fprintf(fid,FMT,data(i,1));
            for j=2:cc(1,2)
                FMT=char(['& %.', num2str(round(dec(j))),'f']);
                fprintf(fid,FMT,data(i,j));
            end
            fprintf(fid,'\\\\ \n ');
            if hl(i)==1 %add horizontal line if requested by user
                fprintf(fid,'\\midrule \n');
            end    
            end
        else
            for i=1:cc(1,1)-1
                if txt(i)==1
                    fprintf(fid,'\n \\multicolumn{%g}{l}{%s} \\\\ \n',cc(1,2)-1,char(itext(cursor)));
                    cursor=cursor+1;
                end
                fprintf(fid,'%s ',char(fcol(i)));
            for j=1:cc(1,2)-1
                FMT=char(['& %.', num2str(round(dec(j))),'f']);
                fprintf(fid,FMT,data(i,j));
            end
            fprintf(fid,'\\\\ \n ');
            if hl(i)==1 %add horizontal line if requested by user
               fprintf(fid,'\\midrule \n');
            end    
            end

            fprintf(fid,'%s\n',' \midrule');
                    fprintf(fid,'MEDIAN');
                    for j=1:cc(1,2)-1*fsup
                    FMT=char(['& %.', num2str(round(dec(j))),'f']);
                    fprintf(fid,FMT,median(data(:,j))); 
                    end
            fprintf(fid,'\\\\ \n ');  
            end 

    case 2
        
            if fsup==0
            for i=1:cc(1,1)-1
                if txt(i)==1
                    fprintf(fid,'\n \\multicolumn{%g}{l}{%s} \\\\ \n',cc(1,2)-1,char(itext(cursor)));
                    cursor=cursor+1;
                end
                FMT=char(['%.', num2str(dec(1)),'f']);
                fprintf(fid,FMT,data(i,1));
            for j=2:cc(1,2)
                FMT=char(['& %.', num2str(round(dec(j))),'f']);
                fprintf(fid,FMT,data(i,j));
            end
            fprintf(fid,'\\\\ \n ');
            if hl(i)==1 %add horizontal line if requested by user
                fprintf(fid,'\\midrule \n');
            end    
            end
        else
            for i=1:cc(1,1)-1
                if txt(i)==1
                    fprintf(fid,'\n \\multicolumn{%g}{l}{%s} \\\\ \n',cc(1,2)-1,char(itext(cursor)));
                    cursor=cursor+1;
                end
                fprintf(fid,'%s ',char(fcol(i)));
            for j=1:cc(1,2)-1
                FMT=char(['& %.', num2str(round(dec(j))),'f']);
                fprintf(fid,FMT,data(i,j));
            end
            fprintf(fid,'\\\\ \n ');
            if hl(i)==1 %add horizontal line if requested by user
               fprintf(fid,'\\midrule \n');
            end    
            end

            fprintf(fid,'%s\n',' \midrule');
                    fprintf(fid,'AVERAGE');
                    for j=1:cc(1,2)-1*fsup
                    FMT=char(['& %.', num2str(round(dec(j))),'f']);
                    fprintf(fid,FMT,mean(data(:,j))); 
                    end
            fprintf(fid,'\\\\ \n ');   
            end
            
        otherwise
            cd ..
              error('TabTeX.m ERROR: Illegal specification in ''stat'' ')
end

% - close table environment, print notes
fprintf(fid,'\n');
fprintf(fid,'\\bottomrule \\addlinespace');
fprintf(fid,'\\end{tabular} \n');
if ~isempty(fnote)
    fprintf(fid,'\\newline \n');
    switch fsty
        case 0
            fprintf(fid,'\\scriptsize Notes: %s\n',char(fnote(1)));
        case 1
            fprintf(fid,'\\scriptsize \\textit{Notes: %s\n}',char(fnote(1)));
        otherwise
            cd ..
            error('TabTeX.m ERROR: Illegal specification in ''fsty'' ')
    end
end
if ~isempty(mpage)
    fprintf(fid,'\\end{minipage}\n');
end
fprintf(fid,'\\end{table} \n');
fclose(fid); %close file
cd .. % go back to current workspace path



